using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using SituationMonitor.Data;

namespace SituationMonitor.Controls
{
    /// <summary>
    /// Class to draw gameboard
    /// </summary>
    public partial class Gameboard : UserControl
    {
        Brush brsBoard = System.Drawing.Brushes.Green;
        Pen linePen = System.Drawing.Pens.Purple;
        Pen lineCustomPen = new Pen(Color.Yellow, 1);
        Pen circlePen = System.Drawing.Pens.White;
        Font textFont = new System.Drawing.Font(FontFamily.GenericSansSerif, 8);
        Brush textBrush = System.Drawing.Brushes.White;

        private static string PLAYER_ACTIVE = "player-active";
        private static string PLAYER_TEAM = "player-team";
        private static string PLAYER_OTHER = "player-other";
        private static string BALL = "ball";
        private static string POINT = "point";
        private static string LINE = "line";
        private static string CIRCLE = "circle";

        private static int BORDER = 5;
        private static int PITCH_LENGTH = 105 + BORDER * 2;
        private static int PITCH_WIDTH = 68 + BORDER * 2;

        public delegate void SelectedSituationChangedHandler(object sender, EventArgs e);
        public event SelectedSituationChangedHandler SelectedSituationChanged;

        [DefaultValue(true)]
        public bool OpenButton
        {
            get { return btnOpen.Visible; }
            set { btnOpen.Visible = value; }
        }

        [Browsable(false)]
        public SituationsDataset Dataset
        {
            get { return dstSituations; }
            set { dstSituations = value; }
        }

        public Gameboard()
        {
            InitializeComponent();
            Draw(GetSelectedSituation());
        }

        /// <summary>
        /// Draws everything with currently selected situation
        /// </summary>
        public void Draw()
        {
            Draw(GetSelectedSituation());
        }

        /// <summary>
        /// Draws everything
        /// </summary>
        private void Draw(SituationsDataset.SituationRow row)
        {
            Bitmap bmp = new Bitmap(pictureBox.Width, pictureBox.Height);
            Graphics graphics = Graphics.FromImage(bmp);
            graphics.Clip = new Region(new RectangleF(0, 0, pictureBox.Width, pictureBox.Height));

            DrawBoard(graphics);
            DrawObjects(graphics, row);

            pictureBox.Image = bmp;
        }

        /// <summary>
        /// Draws board - not used
        /// </summary>
        /// <param name="gr"></param>
        private void DrawBoard(Graphics gr)
        {
        }

        /// <summary>
        /// Draws a player
        /// </summary>
        /// <param name="gr"></param>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="bitmap"></param>
        private void DrawBitmap(Graphics gr, int x, int y, Bitmap bitmap)
        {
            gr.DrawImage(bitmap, x - bitmap.Width/2, gr.ClipBounds.Height - y - bitmap.Height/2);
        }

        /// <summary>
        /// Draws teammate
        /// </summary>
        /// <param name="gr"></param>
        /// <param name="x"></param>
        /// <param name="y"></param>
        private void DrawPlayerTeam(Graphics gr, int x, int y)
        {
            DrawBitmap(gr, x, y, Properties.Resources.RedPlayer);
        }

        /// <summary>
        /// Draws other player
        /// </summary>
        /// <param name="gr"></param>
        /// <param name="x"></param>
        /// <param name="y"></param>
        private void DrawPlayerOther(Graphics gr, int x, int y)
        {
            DrawBitmap(gr, x, y, Properties.Resources.YellowPlayer);
        }

        private void DrawPlayerActive(Graphics gr, int x, int y)
        {
            DrawBitmap(gr, x, y, Properties.Resources.ActivePlayer);
        }

        /// <summary>
        /// Draws ball
        /// </summary>
        /// <param name="gr"></param>
        /// <param name="x"></param>
        /// <param name="y"></param>
        private void DrawBall(Graphics gr, int x, int y)
        {
            DrawBitmap(gr, x, y, Properties.Resources.Ball);
        }

        private void DrawText(Graphics gr, int x, int y, string text)
        {
            gr.DrawString(text, textFont, textBrush, x, gr.ClipBounds.Height - y);
        }

        /// <summary>
        /// Draws a line
        /// </summary>
        /// <param name="gr"></param>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="x2"></param>
        /// <param name="y2"></param>
        private void DrawLine(Graphics gr, int x, int y, int x2, int y2, string  text)
        {
            gr.DrawLine(linePen, x, gr.ClipBounds.Height - y, x2, gr.ClipBounds.Height - y2);
            if (text != null)
                DrawText(gr, x2, y2, text);
        }

        private void DrawLine(Graphics gr, int x, int y, int x2, int y2, string color, string text)
        {
            lineCustomPen.Color = Color.FromName(color);
            gr.DrawLine(lineCustomPen, x, gr.ClipBounds.Height - y, x2, gr.ClipBounds.Height - y2);
            if (text != null)
                DrawText(gr, x2, y2, text);
        }

        private void DrawCircle(Graphics gr, int x, int y, int r1, int r2)
        {
            gr.DrawEllipse(circlePen, x - r1, gr.ClipBounds.Height - y - r2, r1 * 2, r2 * 2);
        }

        private void DrawPoint(Graphics gr, int x, int y)
        {
            DrawBitmap(gr, x, y, Properties.Resources.Point);
        }

        /// <summary>
        /// If type is Player from out team
        /// </summary>
        /// <param name="type"></param>
        /// <returns></returns>
        private bool IsPlayerTeam(string type)
        {
            return type.Equals(PLAYER_TEAM);
        }

        /// <summary>
        /// If type is Player from out team
        /// </summary>
        /// <param name="type"></param>
        /// <returns></returns>
        private bool IsPlayerOther(string type)
        {
            return type.Equals(PLAYER_OTHER);
        }

        private bool IsPlayerActive(string type)
        {
            return type.Equals(PLAYER_ACTIVE);
        }

        private bool IsLine(string type)
        {
            return type.Equals(LINE);
        }

        /// <summary>
        /// Returns true if object is ball
        /// </summary>
        /// <param name="type"></param>
        /// <returns></returns>
        private bool IsBall(string type)
        {
            return type.Equals(BALL);
        }

        private bool IsCircle(string type)
        {
            return type.Equals(CIRCLE);
        }

        private bool IsPoint(string type)
        {
            return type.Equals(POINT);
        }

        /// <summary>
        /// Normalizes value from one range to another
        /// </summary>
        /// <param name="fromRange"></param>
        /// <param name="toRange"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        private int Norm(int fromRange, int toRange, int value)
        {
            return (int)Math.Round((value / (double)fromRange) * toRange);
        }

        /// <summary>
        /// Returns normalized X coordinate
        /// </summary>
        /// <param name="gr"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        private int NormX(Graphics gr, int value)
        {
            return NormWidth(gr, value + PITCH_LENGTH / 2);
        }

        /// <summary>
        /// Returns normalized Y coordinate
        /// </summary>
        /// <param name="gr"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        private int NormY(Graphics gr, int value)
        {
            return NormHeight(gr, - value + PITCH_WIDTH/2);
        }

        private int NormWidth(Graphics gr, int value)
        {
            return Norm(PITCH_LENGTH, (int)Math.Ceiling(gr.ClipBounds.Width), value);
        }

        private int NormHeight(Graphics gr, int value)
        {
            return Norm(PITCH_WIDTH, (int)Math.Ceiling(gr.ClipBounds.Height), value);
        }

        private void DrawObject(Graphics gr, SituationsDataset.ObjectRow r)
        {
            if (IsPlayerOther(r.type))
                DrawPlayerOther(gr, NormX(gr, r.x), NormY(gr, r.y));
            else if (IsPlayerTeam(r.type))
                DrawPlayerTeam(gr, NormX(gr, r.x), NormY(gr, r.y));
            else if (IsBall(r.type))
                DrawBall(gr, NormX(gr, r.x), NormY(gr, r.y));
            else if (IsLine(r.type))
            {
                if (r.IscolorNull())
                    DrawLine(gr, NormX(gr, r.x), NormY(gr, r.y), NormX(gr, r.x2), NormY(gr, r.y2), (r.IsvalueNull() ? null : r.value.ToString()));
                else
                    DrawLine(gr, NormX(gr, r.x), NormY(gr, r.y), NormX(gr, r.x2), NormY(gr, r.y2), r.color, (r.IsvalueNull() ? null : r.value.ToString()));
            }
            else if (IsCircle(r.type))
                DrawCircle(gr, NormX(gr, r.x), NormY(gr, r.y), NormWidth(gr, r.radius), NormHeight(gr, r.radius));
            else if (IsPoint(r.type))
                DrawPoint(gr, NormX(gr, r.x), NormY(gr, r.y));
            else if (IsPlayerActive(r.type))
                DrawPlayerActive(gr, NormX(gr, r.x), NormY(gr, r.y));
        }

        /// <summary>
        /// Draws all objects
        /// </summary>
        /// <param name="gr"></param>
        /// <param name="row"></param>
        private void DrawObjects(Graphics gr, SituationsDataset.SituationRow row)
        {
            if (row != null && gr != null)
            {
                SituationsDataset.ObjectRow[] rows = row.GetObjectRows();
                foreach (SituationsDataset.ObjectRow r in rows)
                    DrawObject(gr, r);
            }
        }

        /// <summary>
        /// Loads situations to combobox
        /// </summary>
        private void LoadSituations(string filter)
        {
            cmbSituation.Items.Clear();
            int i = 0;

            DataView view = new DataView(dstSituations.Situation, filter, "", DataViewRowState.CurrentRows);

            foreach(DataRowView row in view)
            {
                i++;
                cmbSituation.Items.Add(new Data.ComboboxDataholder(row.Row, new string[] { "type", "time" }, "Situation '{0}' #" + i.ToString() + " in time {1}"));
            }
        }

        /// <summary>
        /// Opens dialog window to open situations file
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOpen_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            bool ok = true;

            if (dlg.ShowDialog() == DialogResult.OK)
            {
                dstSituations.Clear();
                try
                {
                    dstSituations.ReadXml(dlg.FileName, XmlReadMode.IgnoreSchema);
                }
                catch (Exception ex)
                {
                    ok = false;
                    MessageBox.Show("Can not read XML file: \n" + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                LoadSituations("");
                if (ok)
                    MessageBox.Show("Nr. of situations loaded: #" + dstSituations.Situation.Rows.Count.ToString(), "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            dlg.Dispose();
        }

        /// <summary>
        /// Returns selected situation
        /// </summary>
        /// <returns></returns>
        public SituationsDataset.SituationRow GetSelectedSituation()
        {
            Data.ComboboxDataholder holder = cmbSituation.SelectedItem as Data.ComboboxDataholder;
            if (holder != null)
                return holder.Row as Data.SituationsDataset.SituationRow;
            else
                return null;
        }

        /// <summary>
        /// Situation selection
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmbSituation_SelectedIndexChanged(object sender, EventArgs e)
        {
            Data.SituationsDataset.SituationRow row = GetSelectedSituation();
            if (row != null)
                Draw(row);
            RaiseSelectionChangedEvent();
        }

        /// <summary>
        /// Filter by exact time from another situation
        /// </summary>
        /// <param name="row"></param>
        public void FilterTime(SituationsDataset.SituationRow row)
        {
            if (row != null)
                LoadSituations("time = " + row["time"].ToString());
            else
                LoadSituations("time = null");
        }

        /// <summary>
        /// Raises event of selection change
        /// </summary>
        private void RaiseSelectionChangedEvent()
        {
            if (SelectedSituationChanged != null)
                SelectedSituationChanged(this, new EventArgs());
        }
    }
}
